import re
import cv2
import pytesseract
from PIL import Image
from datetime import datetime
from dateutil.parser import parse
import tempfile
import uuid


def extract_driving_licence_number(input):
    regex = r"[A-Z]{2}[-\s]\d{13}|[A-Z]{2}[0-9]{2}\s[0-9]{11}"
    match = re.search(regex, input)
    driving_licence_number = match.group(0) if match else ""

    return driving_licence_number


def extract_all_dates(input):
    regex = r"\b\d{2}[/-]\d{2}[/-]\d{4}\b"
    dates = re.findall(regex, input)
    dates = sorted(dates, key=lambda x: int(re.split(r"[-/]", x)[-1]))

    seen = set()
    unique_dates = []

    for date in dates:
        if date not in seen:
            seen.add(date)
            unique_dates.append(date)

    dob = unique_dates[0]
    doi = [unique_dates[1]] if len(unique_dates) > 1 else []
    validity = []

    year = int(re.split(r"[-/]", unique_dates[1])[-1]) if len(unique_dates) > 1 else -1
    validity_duration = 8

    i = 2
    while i < len(unique_dates):
        curr_year = int(re.split(r"[-/]", unique_dates[i])[-1])
        if curr_year - year <= validity_duration:
            doi.append(unique_dates[i])
        else:
            break
        i += 1

    while i < len(unique_dates):
        validity.append(unique_dates[i])
        i += 1

    return dob, doi, validity


def clean_input(match):
    cleaned = []

    for name in match:
        split_name = name.split("\n")
        for chunk in split_name:
            cleaned.append(chunk)

    return cleaned


def extract_all_names(input):
    regex = r"\b[A-Z\s]+\b"
    match = re.findall(regex, input)

    names = []
    cleaned = clean_input(match)

    stopwords = [
        "INDIA",
        "OF",
        "UNION",
        "PRADESH",
        "TRANSPORT",
        "DRIVING",
        "LICENCE",
        "FORM",
        "MCWG",
        "LMV",
        "TRANS",
        "ANDHRA",
        "UTTAR",
        "MAHARASHTRA",
        "GUJARAT",
        "TAMIL",
        "NADU",
        "WEST",
        "BENGAL",
        "KERELA",
        "KARNATAKA",
        "DRIVE",
        "AUTHORIZATION",
        "FOLLOWING",
        "CLASS",
        "DOI",
    ]

    names = [
        name.strip()
        for name in cleaned
        if not any(word in name for word in stopwords) and len(name.strip()) > 3
    ]

    return names


def extract_address_regex(input):
    regex_list = [
        r"Address\s*:\s*\n*(.*?)(?=\n\n|\Z)",
        r"Add\b\s*(.*?)(?=\bPIN|$)",
        r"Address\b\s*(.*?)(?=\bPIN|$)",
        r"Address\s*:\s*((?:(?!(?:Valid Till)).*(?:\n|$))+)",
        r"ADDRESS - (.+?)(?= (?:\b\d{6}\b|$))",
        r"Address\s*:\s*((?:(?!(?:Valid Till)).*(?:\n|$))+)",
    ]
    regex = "|".join(regex_list)

    matches = re.findall(regex, input, re.DOTALL)

    address = ""
    found = 0
    for match in matches:
        for group in match:
            if group:
                address = group.strip()
                found = 1
                break
        if found:
            break

    return address


def extract_address(image_path):
    image = Image.open(image_path)
    text = pytesseract.image_to_string(image)

    if "Add" not in text and "ADD" not in text:
        return ""
    rgb = image.convert("RGB")
    with tempfile.TemporaryDirectory() as tempdir:
        tempfile_path = f"{tempdir}/{str(uuid.uuid4())}.jpg"
        rgb.save(tempfile_path)

        image = cv2.imread(tempfile_path)

        gray_image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)

        config = r"--oem 3 --psm 6"
        boxes_data = pytesseract.image_to_data(gray_image, config=config)

        boxes = boxes_data.splitlines()
        boxes = [b.split() for b in boxes]

        left, top = -1, -1
        for box in boxes[1:]:
            if len(box) == 12:
                if "Add" in box[11] or "ADD" in box[11]:
                    left = int(box[6])
                    top = int(box[7])

        if left == -1:
            return extract_address_regex(text)

        h, w = gray_image.shape

        right = min(left + int(0.4 * w), w)
        bottom = min(top + int(0.18 * h), h)

        roi = gray_image[top:bottom, left:right]
        address = pytesseract.image_to_string(roi, config=config)

        split_address = address.split(" ")
        split_address.remove(split_address[0])

        address = " ".join(split_address)

        return address


def extract_auth_allowed(input):
    auth_types, auth_allowed = [
        "MCWG",
        "M.CYL.",
        "LMV-NT",
        "LMV",
        "TRANS",
        "INVCRG",
    ], []

    for auth in auth_types:
        if auth in input:
            auth_allowed.append(auth)

    return auth_allowed


def expired(input):
    try:
        date = parse(input, dayfirst=True)
        curr = datetime.now()
        diff = date - curr
        if diff.days >= 0:
            return False
        return True
    except:
        return False


def extract_driving_license_details(image_path):
    image = Image.open(image_path)

    extracted_text = pytesseract.image_to_string(image)

    names = extract_all_names(extracted_text)
    full_name = names[0] if len(names) > 0 else ""
    swd_name = names[1] if len(names) > 1 else ""

    dob, issue_date, validity = extract_all_dates(extracted_text)

    driving_licence_number = extract_driving_licence_number(extracted_text)

    authorizations = extract_auth_allowed(extracted_text)

    address = extract_address(image_path)

    remark = ""
    if len(validity) and expired(validity[-1]):
        remark = "The driving licence has been expired."

    return {
        "Full Name": full_name,
        "S/W/D": swd_name,
        "Date of Birth": dob,
        "Issue Date": issue_date,
        "Validity": validity,
        "Driving Licence Number": driving_licence_number,
        "Authorizations": authorizations,
        "Address": address,
        "Remark": remark,
    }


def driving_licence(image_path):
    return extract_driving_license_details(image_path)



import re
import pytesseract
import imghdr
from PIL import Image


def clean_input(match):
    cleaned = []

    for name in match:
        split_name = name.split("\n")
        for chunk in split_name:
            cleaned.append(chunk)

    return cleaned


def extract_all_names(input):
    regex = r"\n[A-Z\s]+\b"
    match = re.findall(regex, input)

    names = []
    cleaned = clean_input(match)

    stopwords = ["INDIA", "OF", "TAX", "GOVT", "DEPARTMENT", "INCOME"]

    names = [
        name.strip()
        for name in cleaned
        if not any(word in name for word in stopwords) and len(name.strip()) > 3
    ]

    return names


def extract_pan(input):
    regex = r"[A-Z]{5}[0-9]{4}[A-Z]"
    match = re.search(regex, input)
    pan_number = match.group(0) if match else ""

    return pan_number


def extract_dob(input):
    """
    # This regex pattern matches dates in dd/mm/yyyy,
    dd-mm-yyyy, dd.mm.yyyy, and dd/mm/yy formats.
    # It accommodates dates separated by slashes (/),
    hyphens (-), or dots (.), and years in both four-digit
    and two-digit formats.
    """

    regex = r"\b(\d{2}[/\-.]\d{2}[/\-.](?:\d{4}|\d{2}))\b"
    match = re.search(regex, input)
    dob = match.group(0) if match else ""

    return dob


def extract_pan_details(image_path):
    image = Image.open(image_path)
    extracted_text = pytesseract.image_to_string(image)

    format = imghdr.what(image_path)
    if format != "jpeg":
        image.save("image.jpg", "JPEG")

        converted_image = Image.open("image.jpg")
        converted_image_text = pytesseract.image_to_string(converted_image)

        extracted_text += converted_image_text

    names = extract_all_names(extracted_text)
    full_name = names[0] if len(names) > 0 else ""
    parents_name = names[1] if len(names) > 1 else ""
    dob = extract_dob(extracted_text)
    pan_number = extract_pan(extracted_text)

    return {
        "Full Name": full_name,
        "Parent's Name": parents_name,
        "Date of Birth": dob,
        "PAN Number": pan_number,
    }


def pan(image_path):
    return extract_pan_details(image_path)



from openbharatocr.ocr.pan import pan
from openbharatocr.ocr.aadhaar import front_aadhaar, back_aadhaar
from openbharatocr.ocr.driving_licence import driving_licence
from openbharatocr.ocr.passport import passport
from openbharatocr.ocr.voter_id import voter_id_front, voter_id_back
from openbharatocr.ocr.vehicle_registration import vehicle_registration
from openbharatocr.ocr.water_bill import water_bill



from openbharatocr.ocr.driving_licence import *
import pytest
from unittest import TestCase
import tempfile
from datetime import datetime, timedelta
import os
from PIL import UnidentifiedImageError


class Test_extract_driving_licence_number(TestCase):
    def test_extract_driving_licence_number_valid_cases(self):
        # Test case 1: Valid driving license number with hyphen
        input_text = (
            "This is a sample text with driving license number DL-1420110012345"
        )
        expected_output1 = "DL-1420110012345"
        assert extract_driving_licence_number(input_text) == expected_output1

        # Test case 2: Valid driving license number with space
        input_text = (
            "This is another sample text with driving license number DL14 20110012345"
        )
        expected_output2 = "DL14 20110012345"
        assert extract_driving_licence_number(input_text) == expected_output2

    def test_extract_driving_licence_number_error_cases(self):
        # Test case 1: Input is None
        input_text = None
        with pytest.raises(TypeError):
            extract_driving_licence_number(input_text)

        # Test case 2: Input is a non-string type
        input_text = 123
        with pytest.raises(TypeError):
            extract_driving_licence_number(input_text)


class Test_extract_all_dates(TestCase):
    def test_extract_all_dates_valid_cases(self):
        # Test case 1: Input with multiple valid dates
        input_text = "This is a sample text with dates 01/01/2000, 15/06/2010, 30/12/2015, and 01/01/2023."
        expected_dob = "01/01/2000"
        expected_doi = ["15/06/2010", "30/12/2015"]
        expected_validity = ["01/01/2023"]
        assert extract_all_dates(input_text) == (
            expected_dob,
            expected_doi,
            expected_validity,
        )

        # Test case 2: Input with only one valid date
        input_text = "This text has a single date: 25/03/2022."
        expected_dob = "25/03/2022"
        expected_doi = []
        expected_validity = []
        assert extract_all_dates(input_text) == (
            expected_dob,
            expected_doi,
            expected_validity,
        )

    def test_extract_all_dates_error_cases(self):
        # Test case 1: Input is None
        input_text = None
        with pytest.raises(TypeError):
            extract_all_dates(input_text)

        # Test case 2: Input is a non-string type
        input_text = 123
        with pytest.raises(TypeError):
            extract_all_dates(input_text)


class Test_clean_input(TestCase):
    def test_clean_input_valid_cases(self):
        # Test case 1: Input with newline characters
        input_match = ["AMIT\nKUMAR", "RITU\nSINGH"]
        expected_output = ["AMIT", "KUMAR", "RITU", "SINGH"]
        assert clean_input(input_match) == expected_output

        # Test case 2: Input without newline characters
        input_match = ["AMIT", "RITU"]
        expected_output = ["AMIT", "RITU"]
        assert clean_input(input_match) == expected_output

    def test_clean_input_error_cases(self):
        # Test case 1: Input is None
        input_match = None
        with pytest.raises(TypeError):
            clean_input(input_match)

        # Test case 2: Input is a non-list type
        input_match = 123
        with pytest.raises(TypeError):
            clean_input(input_match)


class Test_extract_all_names(TestCase):
    def test_extract_all_names_valid_cases(self):
        # Test case 1: Input with valid names and stopwords
        input_text = "AMIT KUMAR INDIA TRANSPORT LICENCE RITU SIGNH"
        expected_output = []
        assert extract_all_names(input_text) == expected_output

        # Test case 2: Input without any stopwords
        input_text = "AMIT KUMAR\nRITU SINGH"
        expected_output = ["AMIT KUMAR", "RITU SINGH"]
        assert extract_all_names(input_text) == expected_output

    def test_extract_all_names_error_cases(self):
        # Test case 1: Input is None
        input_text = None
        with pytest.raises(TypeError):
            extract_all_names(input_text)

        # Test case 2: Input is a non-string type
        input_text = 123
        with pytest.raises(TypeError):
            extract_all_names(input_text)


class Test_extract_address_regex(TestCase):
    def test_extract_address_regex_valid_cases(self):
        # Test case 1: Input with address starting with "Address :"
        input_text = "Address :\nLine 1\nLine 2\n\nOther text"
        expected_output = "Line 1\nLine 2"
        assert extract_address_regex(input_text) == expected_output

        # Test case 2: Input with address starting with "ADDRESS -"
        input_text = "Some text ADDRESS - Line 1, Line 2, 123456 Other text"
        expected_output = "Line 1, Line 2,"
        assert extract_address_regex(input_text) == expected_output

    def test_extract_address_regex_error_cases(self):
        # Test case 1: Input is None
        input_text = None
        with pytest.raises(TypeError):
            extract_address_regex(input_text)

        # Test case 2: Input is a non-string type
        input_text = 123
        with pytest.raises(TypeError):
            extract_address_regex(input_text)


class Test_extract_address(TestCase):
    def test_extract_address_error_cases(self):
        # Test case 1: Invalid image path
        with pytest.raises(FileNotFoundError):
            extract_address("invalid_path.jpg")

        # Test case 2: Invalid image format
        with tempfile.NamedTemporaryFile(suffix=".txt") as temp_file:
            with pytest.raises(Exception):
                extract_address(temp_file.name)


class Test_extract_auth_allowed(TestCase):
    def test_extract_auth_allowed_valid_cases(self):
        # Test case 1: Input with multiple authorized types
        input_text = "This is a sample text with MCWG, LMV, and TRANS authorizations."
        expected_output = ["MCWG", "LMV", "TRANS"]
        assert extract_auth_allowed(input_text) == expected_output

        # Test case 2: Input with a single authorized type
        input_text = "The authorization type is M.CYL."
        expected_output = ["M.CYL."]
        assert extract_auth_allowed(input_text) == expected_output

    def test_extract_auth_allowed_error_cases(self):
        # Test case 1: Input is None
        input_text = None
        with pytest.raises(TypeError):
            extract_auth_allowed(input_text)

        # Test case 2: Input is a non-string type
        input_text = 123
        with pytest.raises(TypeError):
            extract_auth_allowed(input_text)


class Test_expired(TestCase):
    def test_expired_valid_cases(self):
        # Test case 1: Future date (not expired)
        future_date = (datetime.now() + timedelta(days=30)).strftime("%d/%m/%Y")
        assert expired(future_date) is False

        # Test case 2: Past date (expired)
        past_date = (datetime.now() - timedelta(days=30)).strftime("%d/%m/%Y")
        assert expired(past_date) is True

    def test_expired_error_cases(self):
        # Test case 1: Input is None
        input_date = None
        assert expired(input_date) is False

        # Test case 2: Input is an invalid date string
        invalid_date = "Invalid Date"
        assert expired(invalid_date) is False


class Test_extract_extract_driving_license_details(TestCase):
    def test_extract_driving_license_details_invalid_path(self):
        # Test case 1: Invalid image path
        invalid_path = "invalid_path.jpg"
        with pytest.raises(FileNotFoundError):
            extract_driving_license_details(invalid_path)

    def test_extract_driving_license_details_invalid_file(self):
        # Test case 2: Invalid file (non-existent or invalid format)
        invalid_path = "invalid_file.txt"
        invalid_format_path = os.path.join(
            os.path.dirname(__file__), "invalid_format.txt"
        )

        # Create an empty file for testing invalid format
        with open(invalid_format_path, "w"):
            pass

        try:
            with pytest.raises((FileNotFoundError, UnidentifiedImageError)):
                extract_driving_license_details(invalid_path)
            with pytest.raises(UnidentifiedImageError):
                extract_driving_license_details(invalid_format_path)
        finally:
            os.remove(invalid_format_path)
